package java.time.zone;

import java.util.Collections;
import java.util.Arrays;
import java.util.List;
import java.time.Duration;
import java.time.Instant;
import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.util.Objects;
import java.time.ZoneOffset;
import java.time.LocalDateTime;
import java.io.Serializable;

public final class ZoneOffsetTransition implements Comparable<ZoneOffsetTransition>, Serializable
{
    private static final long serialVersionUID = -6946044323557704546L;
    private final LocalDateTime transition;
    private final ZoneOffset offsetBefore;
    private final ZoneOffset offsetAfter;
    
    public static ZoneOffsetTransition of(LocalDateTime obj, ZoneOffset obj2, ZoneOffset zoneOffset) {
        Objects.requireNonNull(obj, "transition");
        Objects.requireNonNull(obj2, "offsetBefore");
        Objects.requireNonNull(zoneOffset, "offsetAfter");
        if (obj2.equals(zoneOffset)) {
            throw new IllegalArgumentException("Offsets must not be equal");
        }
        if (obj.getNano() != 0) {
            throw new IllegalArgumentException("Nano-of-second must be zero");
        }
        return new ZoneOffsetTransition(obj, obj2, zoneOffset);
    }
    
    ZoneOffsetTransition(LocalDateTime transition, ZoneOffset offsetBefore, ZoneOffset offsetAfter) {
        this.transition = transition;
        this.offsetBefore = offsetBefore;
        this.offsetAfter = offsetAfter;
    }
    
    ZoneOffsetTransition(long epochSecond, ZoneOffset zoneOffset, ZoneOffset offsetAfter) {
        this.transition = LocalDateTime.ofEpochSecond(epochSecond, 0, zoneOffset);
        this.offsetBefore = zoneOffset;
        this.offsetAfter = offsetAfter;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    private Object writeReplace() {
        return new Ser((byte)2, this);
    }
    
    void writeExternal(DataOutput dataOutput) throws IOException {
        Ser.writeEpochSec(this.toEpochSecond(), dataOutput);
        Ser.writeOffset(this.offsetBefore, dataOutput);
        Ser.writeOffset(this.offsetAfter, dataOutput);
    }
    
    static ZoneOffsetTransition readExternal(DataInput dataInput) throws IOException {
        long epochSec = Ser.readEpochSec(dataInput);
        ZoneOffset offset = Ser.readOffset(dataInput);
        ZoneOffset offset2 = Ser.readOffset(dataInput);
        if (offset.equals(offset2)) {
            throw new IllegalArgumentException("Offsets must not be equal");
        }
        return new ZoneOffsetTransition(epochSec, offset, offset2);
    }
    
    public Instant getInstant() {
        return this.transition.toInstant(this.offsetBefore);
    }
    
    public long toEpochSecond() {
        return this.transition.toEpochSecond(this.offsetBefore);
    }
    
    public LocalDateTime getDateTimeBefore() {
        return this.transition;
    }
    
    public LocalDateTime getDateTimeAfter() {
        return this.transition.plusSeconds((long)this.getDurationSeconds());
    }
    
    public ZoneOffset getOffsetBefore() {
        return this.offsetBefore;
    }
    
    public ZoneOffset getOffsetAfter() {
        return this.offsetAfter;
    }
    
    public Duration getDuration() {
        return Duration.ofSeconds((long)this.getDurationSeconds());
    }
    
    private int getDurationSeconds() {
        return this.getOffsetAfter().getTotalSeconds() - this.getOffsetBefore().getTotalSeconds();
    }
    
    public boolean isGap() {
        return this.getOffsetAfter().getTotalSeconds() > this.getOffsetBefore().getTotalSeconds();
    }
    
    public boolean isOverlap() {
        return this.getOffsetAfter().getTotalSeconds() < this.getOffsetBefore().getTotalSeconds();
    }
    
    public boolean isValidOffset(ZoneOffset zoneOffset) {
        return !this.isGap() && (this.getOffsetBefore().equals(zoneOffset) || this.getOffsetAfter().equals(zoneOffset));
    }
    
    List<ZoneOffset> getValidOffsets() {
        if (this.isGap()) {
            return Collections.emptyList();
        }
        return Arrays.asList(new ZoneOffset[] { this.getOffsetBefore(), this.getOffsetAfter() });
    }
    
    @Override
    public int compareTo(ZoneOffsetTransition zoneOffsetTransition) {
        return this.getInstant().compareTo(zoneOffsetTransition.getInstant());
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if ((o instanceof ZoneOffsetTransition)) {
            ZoneOffsetTransition zoneOffsetTransition = (ZoneOffsetTransition)o;
            return this.transition.equals(zoneOffsetTransition.transition) && this.offsetBefore.equals(zoneOffsetTransition.offsetBefore) && this.offsetAfter.equals(zoneOffsetTransition.offsetAfter);
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        return this.transition.hashCode() ^ this.offsetBefore.hashCode() ^ Integer.rotateLeft(this.offsetAfter.hashCode(), 16);
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("Transition[").append(this.isGap() ? "Gap" : "Overlap").append(" at ").append(this.transition).append(this.offsetBefore).append(" to ").append(this.offsetAfter).append(']');
        return sb.toString();
    }
}
